import TilemetaUtils from './support/TilemetaUtils';
import { getMapModules } from '../../core/loadMapModules';

export default function ({ TileLayer, lang, promiseUtils } = getMapModules()) {
  var dpi = 96, //pixel per inch
    level = 99, //地图缩放级数
    maxlevel = 18, //地图总级数
    levelStart = 3, // 起始级数
    resolution_0 = 156543.0339280234, //第0级分辨率 米/像素
    tileSize = 256, //瓦片大小
    format = 'PNG', //图片格式
    wkid = 4526, //102113
    fullExtent = [705289.3147226945, 206299.30564637203, 19490490.956488788, 8865103.18739793], //[-20037508.3427892,-20037508.3427892,20037508.3427892,20037508.3427892]
    origin = { x: -20037508.3427892, y: 20037508.3427892 }, //裁图原点
    server = {
      //出图服务参数
      count: 4, //服务总数
      start: 1, //服务起始编号
      seq: 0, //当前请求编号
      //矢量
      'amap-v':
        window.location.protocol +
        '//webrd0{num}.is.autonavi.com/appmaptile?lang=zh_cn&size=1&scale={scale}&style=8&x={col}&y={row}&z={level}&type=map',
      //影像
      'amap-i':
        window.location.protocol +
        '//webst0{num}.is.autonavi.com/appmaptile?style=6&x={col}&y={row}&z={level}&type=map',
      //影像注记
      'amap-ia':
        window.location.protocol +
        '//webst0{num}.is.autonavi.com/appmaptile?x={col}&y={col}&z={level}&lang=zh_cn&size=1&scale={scale}&style=8&type=map',
      //矢量x2
      'amap-v2':
        window.location.protocol +
        '//webrd0{num}.is.autonavi.com/appmaptile?lang=zh_cn&size=1&scale={scale}&style=8&x={col}&y={row}&z={level}&scl=2&type=map',
      //影像x2
      'amap-i2':
        window.location.protocol +
        '//webst0{num}.is.autonavi.com/appmaptile?style=6&x={col}&y={row}&z={level}&scl=2&type=map',
      //影像注记x2
      'amap-ia2':
        window.location.protocol +
        '//webst0{num}.is.autonavi.com/appmaptile?x={col}&y={col}&z={level}&lang=zh_cn&size=1&scale={scale}&style=8&type=map',
    };
  return TileLayer.createSubclass([], {
    declaredClass: 'gis.layers.AMapTileLayer',
    _fetchService: function () {
      //重写 获取元数据，读取本地TileMetaData
      return promiseUtils.resolve().then(
        function () {
          this.read(
            TilemetaUtils.getTilemeta(
              resolution_0,
              dpi,
              level,
              levelStart,
              wkid,
              fullExtent,
              origin,
              tileSize,
              format,
            ),
          );
        }.bind(this),
      );
    },
    getTileUrl: function (level, row, col) {
      //重写 获取瓦片服务地址
      server.seq++, (server.seq %= server.count);
      return maxlevel < level
        ? null
        : lang.replace(server[this.layerType], {
            num: server.seq + server.start,
            row: row,
            col: col,
            level: level,
            scale:
              window.devicePixelRatio > 1
                ? window.devicePixelRatio
                : Math.ceil(window.devicePixelRatio),
          });
    },
  });
}
