// 自定义底图 (由TileLayer扩展)
// create by zxc 2020/7/16
import TilemetaUtils from './support/TilemetaUtils';
import { getMapModules } from '../../core/loadMapModules';

export default function ({ TileLayer, lang, promiseUtils, request } = getMapModules()) {
  var dpi = 96, //pixel per inch
    level = 99, //地图缩放级数
    levelStart = 0, // 起始级数
    maxlevel = 20, //地图总级数
    resolution_0 = 156543.033880145821696, //第0级分辨率 米/像素 156543.033880145821696
    tileSize = 256, //瓦片大小
    format = 'PNG', //图片格式
    wkid = 4526, //102113
    fullExtent = [-20037508.3427892, -20037508.3427892, 20037508.3427892, 20037508.3427892], //[-20037508.3427892,-20037508.3427892,20037508.3427892,20037508.3427892]
    origin = {
      x: -20037508.3427892,
      y: 20037508.3427892,
    }, //裁图原点
    server = {
      //出图服务参数
      'pipenet-tile':
        window.location.protocol +
        '//{url}/gwc/service/wmts?layer={baseLayer}&tilematrixset={tileMatrix}&Service=WMTS&Request=GetTile&Version=1.0.0&Format=image%2Fpng&TileMatrix={tileMatrix}%3A{level}&TileCol={col}&TileRow={row}',
    };
  return TileLayer.createSubclass([], {
    declaredClass: 'gis.layers.PipenetTileLayer',
    _fetchService: function () {
      //重写 获取元数据，直接读取本地TileMetaData
      return promiseUtils.resolve().then(
        function () {
          if (this.extent && this.extent.split(',').length == 4)
            fullExtent = [
              parseFloat(this.extent.split(',')[0]),
              parseFloat(this.extent.split(',')[1]),
              parseFloat(this.extent.split(',')[2]),
              parseFloat(this.extent.split(',')[3]),
            ];
          if (this.levelStart && parseFloat(this.levelStart))
            levelStart = parseFloat(this.levelStart);
          if (this.resolution && parseFloat(this.resolution))
            resolution_0 = parseFloat(this.resolution);
          if (this.origin && this.origin.split(',').length == 2)
            origin = {
              x: parseFloat(this.origin.split(',')[0]),
              y: parseFloat(this.origin.split(',')[1]),
            };
          this.read(
            TilemetaUtils.getTilemeta(
              resolution_0,
              dpi,
              level,
              levelStart,
              wkid,
              fullExtent,
              origin,
              tileSize,
              format,
            ),
          );
        }.bind(this),
      );
    },
    getTileUrl: function (level, row, col) {
      //重写 获取瓦片服务地址
      return lang.replace(server[this.layerType], {
        row: row,
        col: col,
        level: level,
        baseLayer: this.baseLayer,
        url: this.url,
        tileMatrix: this.tileMatrix ? this.tileMatrix : 'EPSG102113',
      });
    },
    fetchTile: function (level, row, col) {
      let url = this.getTileUrl(level, row, col);
      let isNeedImg = _.isUndefined(this.isNeedImg) ? true : !!this.isNeedImg;
      if (isNeedImg) {
        var imgSrc = this.fillImgSrc || 'assets/images/webgis/常用工具/透明.png';
        var img = new Image();
        img.src = imgSrc;
        img.width = img.height = tileSize;
      }
      return request(url, {
        responseType: 'image',
        cachBust: true,
      })
        .then(
          function (response) {
            if (response.data) return response.data;
            if (isNeedImg) return img;
            return null;
          }.bind(this),
        )
        .otherwise(
          function (error) {
            if (isNeedImg) return img;
            return null;
          }.bind(this),
        );
    },
  });
}
