// 自定义底图 (由TileLayer扩展)
// create by ysq 2019/5/8
import TilemetaUtils from './support/TilemetaUtils';
import { getMapModules } from '../../core/loadMapModules';

export default function ({ TileLayer, lang, promiseUtils } = getMapModules()) {
  var dpi = 96, //pixel per inch
    level = 99, //地图缩放级数
    levelStart = 0, // 起始级数
    maxlevel = 20, //地图总级数
    resolution_0 = 156543.033880145821696, //第0级分辨率 米/像素 156543.033880145821696
    tileSize = 256, //瓦片大小
    format = 'PNG', //图片格式
    wkid = 4526, //102113
    fullExtent = [-20037508.3427892, -20037508.3427892, 20037508.3427892, 20037508.3427892], //[-20037508.3427892,-20037508.3427892,20037508.3427892,20037508.3427892]
    origin = {
      x: -20037508.3427892,
      y: 20037508.3427892,
    }, //裁图原点
    server = {
      //出图服务参数
      count: 4, //服务总数
      start: 0, //服务起始编号
      seq: 0, //当前请求编号
      //自定义
      'google-user':
        window.location.protocol +
        '//{url}/gwc/service/wmts?layer={baseLayer}&style=&tilematrixset={tileMatrix}&Service=WMTS&Request=GetTile&Version=1.0.0&Format=image%2Fpng&TileMatrix={tileMatrix}%3A{level}&TileCol={col}&TileRow={row}',
    };

  return TileLayer.createSubclass([], {
    declaredClass: 'gis.layers.UserTileLayer',
    _fetchService: function () {
      //重写 获取元数据，直接读取本地TileMetaData
      return promiseUtils.resolve().then(
        function () {
          if (this.extent !== '' && this.extent.split(',').length == 4)
            fullExtent = [
              parseFloat(this.extent.split(',')[0]),
              parseFloat(this.extent.split(',')[1]),
              parseFloat(this.extent.split(',')[2]),
              parseFloat(this.extent.split(',')[3]),
            ];
          if (this.levelStart !== '' && parseFloat(this.levelStart))
            levelStart = parseFloat(this.levelStart);
          if (this.levelEnd && this.levelEnd == this.levelStart) {
            level = parseInt(this.levelStart) + 1;
            if (_config.client != 'sandbox')
              mapManager.view.on('drag', (e) => {
                e.stopPropagation();
              });
          } else if (this.levelEnd) {
            level = parseInt(this.levelEnd) + 1;
          } else {
          }
          if (this.resolution !== '' && parseFloat(this.resolution))
            resolution_0 = parseFloat(this.resolution);
          if (this.origin !== '' && this.origin.split(',').length == 2)
            origin = {
              x: parseFloat(this.origin.split(',')[0]),
              y: parseFloat(this.origin.split(',')[1]),
            };
          this.read(
            TilemetaUtils.getTilemeta(
              resolution_0,
              dpi,
              level,
              levelStart,
              wkid,
              fullExtent,
              origin,
              tileSize,
              format,
            ),
          );
        }.bind(this),
      );
    },
    getTileUrl: function (level, row, col) {
      //重写 获取瓦片服务地址
      server.seq++, (server.seq %= server.count);
      return lang.replace(server[this.layerType], {
        num: server.seq + server.start,
        row: row,
        col: col,
        level: level,
        url: this.url,
        baseLayer: this.baseLayer,
        scale:
          window.devicePixelRatio > 1
            ? window.devicePixelRatio
            : Math.ceil(window.devicePixelRatio),
        tileMatrix: this.tileMatrix ? this.tileMatrix : 'EPSG:900913',
      });
    },
  });
}
